/*
 * curudlr.c
 * contains: curlf(),currt(),curup(),curdn()
 *
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 */

#include <stdio.h>
#include "gfuncts.h"

/*
 *  int
 * curlf(distance)
 *
 * ARGUMENT
 *  (int)	distance	-	number of positions to move
 *
 * DESCRIPTION
 *  Moves cursor left (distance) positions or to the left edge of the screen
 *  if current position - distance is less than 0.
 *
 * RETURNS
 *  TRUE if the cursor could be moved the requested distance.  FALSE if
 *  the cursor could not be moved the required distance.
 *
 * AUTHOR
 *  ""   10-APR-1987  08:43:09.27
 */
int GF_CONV curlf(distance)
int distance;
{
	unsigned curpos;
	int xpos;
	
	curpos=_gcurvpc();
	xpos=(int)(curpos&0x00ff)-distance;
	_cursetc((int)(curpos>>8)&0x00ff,(xpos<0)?0:xpos);
	return((xpos<0)?FALSE:TRUE);
}

/*
 *  int
 * currt(distance)
 *
 * ARGUMENT
 *  (int)	distance	-	number of positions to move
 *
 * DESCRIPTION
 *  Moves cursor right (distance) positions or to the rightmost column of the 
 *  screen if current position + distance is greater than gmaxcol.
 *
 * RETURNS
 *  TRUE if the cursor could be moved the requested distance.  FALSE if
 *  the cursor could not be moved the required distance.
 *
 * AUTHOR
 *  ""   10-APR-1987  09:19:41.61
 */
int GF_CONV currt(distance)
int distance;
{
	unsigned curpos;
	int xpos;
	
	if(!_gvmtrp)
		vmode(-1);
	curpos=_gcurvpc();
	xpos=(int)(curpos&0x00ff)+distance;
	_cursetc((int)(curpos>>8)&0x00ff,(xpos>=gmaxcol)?gmaxcol-1:xpos);
	return((xpos>=gmaxcol)?FALSE:TRUE);
}

/*
 *  int
 * curup(distance)
 *
 * ARGUMENT
 *  (int)	distance	-	number of positions to move
 *
 * DESCRIPTION
 *  Moves cursor up (distance) positions or to the top edge of the screen
 *  if current position - distance is less than 0.
 *
 * RETURNS
 *  TRUE if the cursor could be moved the requested distance.  FALSE if
 *  the cursor could not be moved the required distance.
 *
 * AUTHOR
 *  ""   10-APR-1987  08:43:09.27
 */
int GF_CONV curup(distance)
int distance;
{
	unsigned curpos;
	int ypos;
	
	curpos=_gcurvpc();
	ypos=(int)((curpos>>8)&0x00ff)-distance;
	_cursetc((ypos<0)?0:ypos,(int)(curpos&0x00ff));
	return((ypos<0)?FALSE:TRUE);
}

/*
 *  int
 * curdn(distance)
 *
 * ARGUMENT
 *  (int)	distance	-	number of positions to move
 *
 * DESCRIPTION
 *  Moves cursor down (distance) positions or to the bottom row of the screen
 *  if current position + distance is greater than gmaxcol.
 *
 * RETURNS
 *  TRUE if the cursor could be moved the requested distance.  FALSE if
 *  the cursor could not be moved the required distance.
 *
 * AUTHOR
 *  ""   10-APR-1987  08:43:09.27
 */
int GF_CONV curdn(distance)
int distance;
{
	unsigned curpos;
	int ypos;
	
	if(!_gvmtrp)
		vmode(-1);
	curpos=_gcurvpc();
	ypos=(int)((curpos>>8)&0x00ff)+distance;
	_cursetc((ypos>=gmaxrow)?gmaxrow-1:ypos,(int)(curpos&0x00ff));
	return((ypos>=gmaxrow)?FALSE:TRUE);
}
