/*
 * daywk.c
 * contains: daywk()
 *
 */

#include <stdio.h>
#include "gfuncts.h"

struct PERPCAL {
	int ord18[29];
	int lep18[29];
	int ord19[29];
	int lep19[29];
	int ord20[29];
	int lep20[29];
};

/*  Structure to contain 2-dimensional array relating doninican letter 0..6 
 *  for each month to day of week for first day of each month:
 *  x2[month][letter] = day_of_week
 */
struct PERPDAY {
	int x2[12][7];
};

struct PERPCAL _perpcal = { 
	4,3,2,1,0,5,4,3,2,0,6,5,4,2,1,0,6,4,3,2,1,6,5,4,3,1,0,6,5,
	4,3,2,1,6,5,4,3,1,0,6,5,3,2,1,0,5,4,3,2,0,6,5,4,2,1,0,6,4,
	6,5,4,3,2,0,6,5,4,2,1,0,6,4,3,2,1,6,5,4,3,1,0,6,5,3,2,1,0,
	6,5,4,3,1,0,6,5,3,2,1,0,5,4,3,2,0,6,5,4,2,1,0,6,4,3,2,1,6,
	1,6,5,4,3,1,0,6,5,3,2,1,0,5,4,3,2,0,6,5,4,2,1,0,6,4,3,2,1,
	0,6,5,4,2,1,0,6,4,3,2,1,6,5,4,3,1,0,6,5,3,2,1,0,5,4,3,2,0 
};

struct PERPDAY _perpday = { 
	0,6,5,4,3,2,1,  3,2,1,0,6,5,4,  3,2,1,0,6,5,4,
	6,5,4,3,2,1,0,  1,0,6,5,4,3,2,  4,3,2,1,0,6,5,  6,5,4,3,2,1,0,
	2,1,0,6,5,4,3,  5,4,3,2,1,0,6,  0,6,5,4,3,2,1,  3,2,1,0,6,5,4,
	5,4,3,2,1,0,6 
};

/*
 *  int
 * daywk(year,month,day)
 *
 * ARGUMENT
 *  (int)	year		-	year from 1800 to 2099
 *  (int)	month		-	month from 1 to 12
 *  (int)	day		-	day of month 1-31
 *
 * DESCRIPTION
 *   The perpetual calendar shown in the Smithsonian Tables is used; the
 *   Gregorian method employed, and tables are provided for the 19th, 20th
 *   and 21st centuries.
 *
 * RETURNS
 *  day of week, 0=sunday, 1=monday etc.
 *
 * AUTHOR
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 */
int GF_CONV daywk(year,month,day)
int year,month,day;
{
	int century,yr,iyear,letter,day1,day_of_week;
	bool leap;

	century=(year-1800)/100;
	iyear=year % 100;
	leap=isleap(year);
	if(iyear<=28) 
		yr=iyear;
	else if((iyear>28)&&(iyear<57))
		yr=iyear-28;
	else if((iyear>56)&&(iyear<85))
		yr=iyear-56;
	else if((iyear>84))
		yr=iyear-84;
	if((century==0)&&!leap)
		letter=_perpcal.ord18[yr];
	else if((century==0)&&leap)
		letter=_perpcal.lep18[yr];
	else if((century==1)&&!leap)
		letter=_perpcal.ord19[yr];
	else if((century==1)&&leap)
		letter=_perpcal.lep19[yr];
	else if((century==2)&&!leap)
		letter=_perpcal.ord20[yr];
	else if((century==2)&&leap)	
		letter=_perpcal.lep20[yr];
	day1=_perpday.x2[(month-1)][letter];
	day_of_week=((day-1)+day1)%7;
	return (day_of_week);
}
