/*
** filecopy.c
** contains: FileCopy()
*/

#include <stdio.h>
#include <stdlib.h>
#include "gf.h"
#include "gfuncts.h"
#include "timedate.h"

/*
**  int
** FileCopy(char *sourcefile, char *destfile, char *buffer,
**          unsigned int buffersize)
**
** ARGUMENT(s)
**  sourcefile          -       ASCIIZ source file name (path names o.k.)
**  destfile            -       ASCIIZ destination file name (path names o.k.)
**  buffer              -       Points to optional user buffer for fread() &
**                              fwrite().
**  buffersize          -       Size of user buffer or size of buffer to
**                              dynamically allocate inside filecopy().
**
** DESCRIPTION
**  Does a filecopy operation.  Updates the destination with the time/date
**  stamp of the source.  Can either use a buffer supplied by the user
**  or allocate one of it's own depending on parameters supplied.
**
** RETURNS
**
**
** AUTHOR
**  ""   Thu 10-Nov-1988  11:37:24
**   Copyright (C)1988-1990 Greenleaf Software Inc. All Rights Reserved.
**
** MODIFICATIONS
**
**  Patrick Little   23-Oct-1989   3:59:23
**   Moved getfstamp to a point where the file was closed.  This was
**   necessary due to a problem with the way Lattice opens buffered
**   files.
*/
int GF_CONV FileCopy(sourcefile,destfile,buffer,buffersize)
char *sourcefile,*destfile,*buffer;
unsigned int buffersize;
{
        FILE *source, *dest;
        char *filebuffer;
        unsigned int bytesread;
        struct TIMEDATE td;
        int retvalue=0,stat;

#ifndef AZTEC
        if((source=fopen(sourcefile,"rb"))==(FILE *)0)
#else
        if((source=fopen(sourcefile,"r"))==(FILE *)0)
#endif
                return(OPENSOURCEERROR);
#ifndef AZTEC
        if((dest=fopen(destfile,"wb"))==(FILE *)0) {
#else
        if((dest=fopen(destfile,"w"))==(FILE *)0) {
#endif

                fclose(source);
                return(OPENDESTINATIONERROR);
        }
        filebuffer=buffer?buffer:malloc(buffersize);
        if(!filebuffer) {
                fclose(source);
                fclose(dest);
                return(NOTENOUGHMEMORY);
        }
        do {
                bytesread=fread(filebuffer,1,buffersize,source);
                if(bytesread)
                        if(fwrite(filebuffer,1,bytesread,dest)<bytesread)
                                retvalue=CANTWRITEDEST;
        } while((bytesread==buffersize)&&!retvalue);
        fclose(source);
        fclose(dest);
/*
 * Moved getfstamp to a position after the file was closed due to a
 * problem with lattice and fopen
 */

        if( !retvalue ) {
            stat = getfstamp( &td, sourcefile );
            if( stat == 0 )
                putfstamp( &td, destfile );
        }
        if( !buffer )
            free( filebuffer );
        return( retvalue );
}
