/*
 * asgetc.c
 * contains: asgetc(),asstat(),asinit(),asputc(),asputs(),asrts(),asdtr(),
 * contains: asbreak(),asreset()
 *
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 *	 
 */

#define	GFASYNCG

#include <stdio.h>
#include "gfuncts.h"
#include "asports.h"

/*  Absolute I/O addresses for each 8250 UART
 */
unsigned as_port[MAX_ASPORTS]={0x3f8,0x2f8,0x3f8,0x3f8,0x3f8,0x3f8,
	0x3f8,0x3f8 
};

/*  0=no check for CTS or DSR, 1=check and wait per as_wtime[], as_rtime[].
 */
int _as_wcts=1;

/*  Delay in millisec for asputc() and asgetc():
 */
int as_wtime[MAX_ASPORTS]={95,95,95,95,95,95,95,95};

int as_rtime[MAX_ASPORTS]={0,0,0,0,0,0,0,0};

/*
 *  int
 * asgetc(port) 
 *
 * ARGUMENT
 *  (int)	port	=	Port to read, 0==COM1, 1==COM2
 *
 * DESCRIPTION
 *
 *  The indicated port is tested for DSR - if not active within a delay
 *  period, timeout is indicated and the function returns -1.  Next,
 *  the UART receive buffer is tested for full - if no character is
 *  received within a timeout period, -1 is returned.  Else the character
 *  is returned.
 *
 * RETURNS
 *  Character from channel, or -1 if no character was available or DSR was
 *  not high, or there was an error.
 */
int GF_CONV asgetc(port)
int port;
{
		
	return as_getc(as_port[port],as_rtime[port]);
}

/*
 *  unsigned
 * asstat(port)
 *
 * ARGUMENT
 *  (int)	port	=	Port to obtain status of.
 *
 * DESCRIPTION
 *  The specified port is queried for status.
 *
 * RETURNS
 *  Contents of line status register in upper 8 bits and contents of
 *  modem status register in lower 8 bits.
 */
unsigned GF_CONV asstat(port)
int port;
{
	
	return as_stat(as_port[port]);
}

/*
 *  int
 * asinit(port,baud,parity,stopbits,wdlen)
 *
 * ARGUMENT
 *  (int)	port	-	0/1 unless optional boards are installed
 *  (int)	baud	-	110,150,300,...1200,...9600
 *  (int)	parity	-	0=none, 1=ODD, 2=EVEN, 3=stick pairty
 *  (int)	stopbits -	1/2
 *  (int)	wdlen	-	word length: 7 or 8 data bits
 *
 * DESCRIPTION
 *  Initializes operating parameters of the serial port (8250), note
 *  all parameters are range checked.
 *
 * RETURNS
 *  Status (see asstat()) is returned or -1 if argument is out of range.
 */
int GF_CONV asinit(port,baud,parity,stopbits,wdlen)
int port,baud,parity,stopbits,wdlen;
{
	unsigned cbaud,cpar,cstop,cwd,mrgbits;
	
	if (port<0||port>1) 
		return (-1);
	switch (wdlen) {
	case 7:	
		cwd=2;
		break;
	case 8:	
		cwd=3;
		break;
	default: 
		return (-1);
	}
	switch (parity) {
	case 0:	
		cpar=0;		/* none */
		break;
	case 1:	
		cpar=8;		/* odd */
		break;
	case 2:	
		cpar=0x18;	/* even */
		break;
	case 3: 
		cpar=0x28;	/* stick space 0 */
		break;
	case 4: 
		cpar=0x38;	/* stick mark 1 */
		break;
	default: 
		return (-1);
	}
	switch (stopbits) {
	case 1:	
		cstop=0;
		break;
	case 2:	
		cstop=0x04;
		break;
	default: 
		return (-1);
	}
	switch (baud) {
	case 110: 
		cbaud=1047;
		break;
	case 150: 
		cbaud=768;
		break;
	case 300: 
		cbaud=384;
		break;
	case 600: 
		cbaud=192;
		break;
	case 1200: 
		cbaud=96;
		break;
	case 2400: 
		cbaud=48;
		break;
	case 4800: 
		cbaud=24;
		break;
	case 9600: 
		cbaud=12;
		break;
	default: 
		return (-1);
	}
	mrgbits=(cpar|cstop|cwd);	/* merge command bits	*/
	return as_iniw(as_port[port],cbaud,mrgbits,_as_wcts);
}

/*
 *  int
 * asputc(port,c)
 *
 * ARGUMENT
 *  (int)	port	-	Port to send to (0 or 1)
 *  (char)	c	-	character to send
 *
 * DESCRIPTION
 *  The character is transmitted over the indicated RS232 port.
 *
 * RETURNS 
 *  -1 if a problem occurred, else status bits (see "asstat")
 */
int GF_CONV asputc(port,c)
int port;
char c;
{
	
	return as_putc(as_port[port],as_wtime[port],c);
}

/*
 *  int
 * asputs(port,str)
 *
 * ARGUMENT
 *  (int)	port	-	port to send to
 *  (char *)	str	-	NULL terminated string to send
 *
 * DESCRIPTION
 *  Asynchronous RS232 send string
 *
 * RETURNS
 *   -1 if string was not sent correctly, else 0.
 *   In particular, DSR and CTS are tested initially and if not present
 *   an immediate return -1 is taken.  If these are present, each
 *   character is sent, and no exit is possible until all are sent.
 */
int GF_CONV asputs(port,str)
int port;
char *str;
{
	char c;
		
	if((asstat(port)&0x0030)!=0x0030) 
		return (-1);	/* if !DSR or !CTS */
	while(c=*str++) {
		while(asputc(port,c)==-1) 
			;		/* repeat each char  until sent */
	}
	while (asputc(port,(char)0)==-1) 
		;		/* send a terminator byte */
	return 0;		/* when all sent, return */
}

/*
 *  void
 * asrts(port,ctrl)
 *
 * ARGUMENT
 *  (int)	port	-	port number to use
 *  (int)	ctrl	-	1 = Assert, 0 = De-assert
 *
 * DESCRIPTION
 *  Control RTS (request to send) on a port.
 */
void GF_CONV asrts(port,ctrl)
int port,ctrl;
{
	as_rts(as_port[port],ctrl);
}

/*
 *  void
 * asdtr(port,ctrl)
 *
 * ARGUMENT
 *  (int)	port	-	port number to use
 *  (int)	ctrl	-	1 = Assert, 0 = De-assert
 *
 * DESCRIPTION
 *  Control DTR (data terminal ready) on a port
 */
void GF_CONV asdtr(port,ctrl)
int port,ctrl;
{
	as_dtr(as_port[port],ctrl);
}

/*
 *  int
 * asbreak(port)
 *
 * ARGUMENT
 *  (int)	port	-	port number to use
 *
 * DESCRIPTION
 *  Send Break signal on comm port.
 *
 * RETURNS
 *  0 if successful, -1 if error
 */
int GF_CONV asbreak(port)
int port;
{
		
	return as_break(as_port[port],as_wtime[port]);
}

/*
 *  void
 * asreset(port)
 *
 * ARGUMENT
 *  (int)	port	-	port number to use
 *
 * DESCRIPTION
 *  set baud rate to 300, parity to odd, stop bits to 1, word length to 8
 *  DTR & RTS to OFF
 */
void GF_CONV asreset(port)
int port;
{
	as_reset(as_port[port]);
}
