/*
 * gfdays.c
 * contains: dayofyr(),dayr(),dayweek()
 *
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 */

#include <stdio.h>
#include "gfuncts.h"

/*
 *  int
 * dayofyr(month,day,year)
 *
 * ARGUMENT
 *  (int)	month	-	month number 1==jan etc
 *  (int)	day	-	day of month 1..31
 *  (int)	year	-	year number 1980 ..
 *
 * DESCRIPTION
 *  Given integer values for month, day, and year, determine the day of
 *  the year (1..366), taking leap years into account.
 *
 * RETURNS
 *  Integer day of the year
 */
int GF_CONV dayofyr(mo,da,yr)
int mo,da,yr;
{
	int i,day,leap;
	
	static int day_tab[2][13] =
	{ 
		{0,31,28,31,30,31,30,31,31,30,31,30,31},
		{0,31,29,31,30,31,30,31,31,30,31,30,31} 
	};
	
	day=da;
	leap=(int)isleap(yr);
	for(i=1;i<mo;i++) 
		day+=day_tab[leap][i];
	return day;
}

/*
 *  int
 * dayr(void)
 *
 * ARGUMENT
 *  (none)
 *
 * DESCRIPTION
 *  Return day of year, use system time & date
 *
 * RETURNS
 *  Integer day of year 1..366
 *
 * MODIFICATIONS
 *  "" Wed 11-Jan-1989 10:52:46
 *    Modified call to gettime() to call ggettime()
 */
int GF_CONV dayr()
{
	int GF_CONV dayofyr();
	struct TIMEDATE *pd;
	
	pd=ggettime();
	return(dayofyr(pd->month,pd->day,pd->year));
}

/*
 *  int
 * dayweek(month,day,year)
 *
 * ARGUMENT
 *  (int)	month	=	Integer value of month (1..12)
 *  (int)	day	=	Integer value of day (1..31)
 *  (int)	year	=	Integer value of year (1980...)
 *
 * DESCRIPTION
 *  Given the year,month and day, this function returns the integer
 *  value of the day of the week.
 *
 * RETURNS
 *  Integer value of day of the week.  1 == Monday etc.
 */
int GF_CONV dayweek(month,day,year)
int month,day,year;
{
	int jan1,iyear;

	jan1=2;	/*  start counting from Jan 1, 1980 in days	*/
	if(year>1980) {
		for(iyear=1980;iyear<year;iyear++) {
			jan1+=365;
			if(isleap(iyear)) 
				++jan1;
		}
	}
	return((jan1+dayofyr(month,day,year)-1)%7);
}
