/*
 * justify.c
 * contains: justify()
 *
 */

#include <stdio.h>
#include "gfuncts.h"

/*
 *  bool
 * justify(dest,source,width)
 *
 * ARGUMENT
 *  (char *) dest	=	destination string
 *  (char *) source	=	source string
 *  (int) width		=	Number of columns to justify on.  Nominally
 *				for 80 column displays use gjwidth = 80.
 *
 * DESCRIPTION
 *   Copy an input string to a destination string with justification.
 *   Tabs and newlines are acceptable in the text.  Tabs are expanded to
 *   TABLENGTH spaces in the output string.  Text is justified from the
 *   first position of the output string to that specified in the width
 *   parameter.  If the width parameter is omitted or is zero, the global
 *   variable gjwidth.
 *
 * RETURNS
 *  TRUE(1) if successful, FALSE(0) if the input string cannot be justified
 *  on the ouput gmaxcol.
 *
 * AUTHOR
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 *
 * MODIFICATIONS
 *
 */
bool GF_CONV justify(outstr,instr,width)
char *outstr,*instr;
int width;
{
	char c,*p,*po;
	int i,words,spacel,chars,length,rem,cout,outwords,col;
	bool inword,spflag;

	words=chars=length=0;
	p=instr;
	inword=FALSE;
	while (1) {
		c = *p++;
		if(!c)
			break;
		switch(c) {
		case '\n':
		case ' ':
			length++;
			inword=FALSE;
			if(length>width)
				goto _toolong;
			break;

		case '\t':
			length=length+gtabexp;
			chars=chars+gtabexp;
			if(chars>width||length>width)
				goto _toolong;
			inword=FALSE;
			break;

		default:
			chars++;
			length++;
			if(chars>width||length>width)
				goto _toolong;
			if(!inword)
				words++;
			inword=TRUE;
			break;

		}
	}
	spacel=((width-chars)/(words-1));
	rem=((width-chars) % (words-1));
	spflag = FALSE;
	p=instr;
	po=outstr;
	col=cout=outwords=0;
	inword = FALSE;
	while (1) {
		c = *p++;
		if(!c)
			break;
		switch(c) {
		case ' ':
			inword=FALSE;
			if(spflag)
				break;
			spflag=TRUE;
			for(i=1;i<=spacel;i++) {
				*po++=' ';
				cout++;
				if(cout>width)
					goto toolong;
			}
			if(rem) {
				*po++=' ';
				--rem;
				cout++;
				if(cout>width)
					goto toolong;
			}
			if(outwords==(words-1)) {
				while (rem--) {
					*po++=' ';
					cout++;
					if(cout>width)
						goto toolong;
				}
			}
			continue;
		case '\t':
			spflag = FALSE;
			for(i=1;(col % gtabexp),i<=gtabexp; i++) {
				*po++=' ';
				if(cout>width)
					goto toolong;
			}
			if(rem) {
				*po++=' ';
				--rem;
				cout++;
				col++;
				if(cout>width)
					goto toolong;
			}
			inword = FALSE;
			continue;
		default:
			spflag = FALSE;
			*po++=c;
			cout++;
			col++;
			if(!inword)
				outwords++;
			inword=TRUE;
			if(cout>width)
				goto toolong;
			continue;
		}
	}
	*po='\0';
	return TRUE;
toolong:
	*po='\0';
	return FALSE;
_toolong:
	outstr[0] = '\0';
	return FALSE;

}
