/*
 * prints.c
 * contains: prints(),cprints(),jprints(),ceprintf(),jprintf()
 *
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 *
 */

#include <stdio.h>
#include <string.h>
#include "gfuncts.h"

void (GF_CONV *prsfun)(char,int,int) = gputchar;


/*
 *  void
 * prints(string)
 *
 * ARGUMENT
 *  (char *) string	=	pointer to string to display
 *
 * DESCRIPTION
 *  Output a string via putchar() or putcinc() using function pointer
 *  prsfun.  To speed up this function significantly substitue the putcinc
 *  function for the gputchar function.
 *	 
 *  As shipped, this function and the numerous others that call it will
 *  be redirectable but slow.  Making the above change will make it
 *  non-redirectable;  it will use a ROM BIOS call which is quite fast.
 *
 * MODIFICATIONS
 *
 */
void GF_CONV prints(str)
char *str;
{

	while(*str)
		(*prsfun)(*str++,7,0);
}

/*
 *  void
 * gputchar(character)
 *
 * ARGUMENT
 *  (char)	character	-	character to output
 *
 * DESCRIPTION
 *  This function is responsible for outputting a single character via
 *  putchar().  It is used because putchar() is #defined as a macro
 *  in many of the stdio.h's.  We needed a function that we could store
 *  a pointer to in prsfun.
 */
void GF_CONV gputchar(character,dum1,dum2)
char character;
int dum1;
int dum2;
{
	dum1 = dum1;  /* to quiet compiler */
	dum2 = dum2;
	putchar(character);
}

/*
 *  void
 * cprints(cols,string)
 *
 * ARGUMENT
 *  (int)	cols	=	Number of columns to center on
 *  (char *)	string	=	String to be centered and displayed.
 *
 * DESCRIPTION
 *  This is a combination of centers() and prints().
 */
void GF_CONV cprints(cols,str)
int cols;
char *str;
{
	struct GFREGS out;

	vstate(&out);		/* Find out what page */
	curset((getcur(out.cx)&0xff00)>>8,
	    xmin(cols,gmaxcol)/2-(strlen(str)/2),out.cx);
	while(*str)
		putcinc(*str++,NORMAL,out.cx);
}

/*
 *  bool
 * jprints(cols,str)
 *
 * ARGUMENT
 *  (int)	cols	=	Number of columns to justify on
 *  (char *)	str	=	String to be justified and displayed
 *
 * DESCRIPTION
 *  The specified string is input to the justify function.  An internal
 *  buffer receives the output.  The results of justify, if any, are
 *  printed on stdout.  Justification is on the lesser of cols or gmaxcol.
 *
 * RETURNS
 *  Normally returns TRUE.  In cases where the string cannot be justified
 *  returns FALSE.
 */
bool GF_CONV jprints(cols,str)
int cols;
char *str;
{
	char out[84];
	int icols;
	bool result;

	icols=xmin(cols,gmaxcol);
	result=justify(out,str,icols);
	prints(out);
	return result;
}

/*
 *  void
 * ceprintf(cols,format,args)
 *
 * ARGUMENT
 *  (int)	cols	=	Number of columns to center on
 *  (char *)	format	=	Format string for sprintf()
 *  		args	=	Arguments for sprintf()
 *
 * DESCRIPTION
 *  The arguments are formatted by sprintf(), then centered on the lesser
 *  of cols or gmaxcol columns, stored in an internal buffer, and then
 *  output using cprints().
 *
 * MODIFICATIONS
 *
 */

#ifndef	_MSC
void GF_CDECL ceprintf(cols,fmt)
#else
#if	_MSC > 4
void GF_CDECL ceprintf(cols,fmt,...)
#else
void ceprintf(cols,fmt)
#endif
#endif

int cols;
char *fmt;
{
	int icols;
	char buf[135];
	char **value;
	
	value=&fmt;
	icols = xmin(cols,gmaxcol);
	sprintf(buf,fmt,
		value[1],value[2],value[3],value[4],value[5],value[6],
		value[7],value[8],value[9],value[10]
#ifndef _LDATA
		,value[11],value[12],value[13],value[14],value[15],value[16],
		value[17],value[18],value[19],value[20]
#endif
		);

	cprints(icols,buf);
}

/*
 *  bool
 * jprintf(cols,format,args)
 *
 * ARGUMENT
 *  (int)	cols	=	Number of columns to justify on
 *  (char *)	format	=	Format string for sprintf
 *  		args	=	Arguments for sprintf()
 *
 * DESCRIPTION
 *  The arguments are formatted via sprintf() according to the format
 *  string, placed in an internal buffer, and then justified on the lesser of
 *  cols and gmaxcol and then output via prints()
 *
 * RETURNS
 *  TRUE if justification successful, FALSE if unsuccessful
 */

#ifndef	_MSC
bool GF_CDECL jprintf(cols,fmt)
#else
#if	_MSC > 4
bool GF_CDECL jprintf(cols,fmt,...)
#else
bool jprintf(cols,fmt)
#endif
#endif

int cols;
char *fmt;
{
	int icols;
	char buf[135];
	char **value;
	value=&fmt;

	icols=xmin(cols,gmaxcol);
	sprintf(buf,fmt,
		value[1],value[2],value[3],value[4],value[5],value[6],
		value[7],value[8],value[9],value[10]
#ifndef _LDATA
		,value[11],value[12],value[13],value[14],value[15],value[16],
		value[17],value[18],value[19],value[20]
#endif
		);
	return(jprints(icols,buf));
}
