/*
 * rputcinc.c
 * contains: rputcinc()
 *
 */

#include <stdio.h>
#include "gfuncts.h"

/*
 *  void
 * rputcinc(c,bkgnd,fgnd,blink,page)
 *
 * ARGUMENT
 *  (char)	c	-	character to write
 *  (int)	bkgnd	-	background color value (0..7)
 *  (int)	fgnd	-	foreground color value (0..15)
 *  (int)	blink	-	blink, 0==NO, 1==YES
 *  (int)	page	-	video page to write
 *
 * DESCRIPTION
 *   Output one character to Color Display and Increment Cursor to Next
 *   Logical Position.  If in last column of a row, increment to next row.
 *   Use cpch to display character and attribute.  Translate newlines to
 *   CR LF sequences.  Determine cursor position at each newline.
 *   This differs from putcinc in that both background and foreground
 *   colors are specified, and a blink attribute may be added.
 *
 * AUTHOR
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 *
 * MODIFICATIONS
 *
 */
void GF_CONV rputcinc(c,bkgnd,fgnd,blink,page)
char c;
int page,bkgnd,fgnd,blink;
{
	int x,y,where,width;
	struct GFREGS out;

	where=getcur(page);
	x=(where&0x00FF);
	y=(where&0xFF00)>>8;
	vstate(&out);
	width=out.bx;
	switch (c) {
	case 0x0A:
		if(y<gmaxrow)
			++y;
		curset(y,0,page);
		break;
	case 0x0D:
		curset(y,0,page);
		break;
	case '\t':
		if(!x) {
			x=8;
			curset(y,x,page);
		} else
			while(x % 8)
				if(++x<width)
					curset(y,x,page);
		break;
	default:
		cpch(c,(fgnd&0x0F),((bkgnd&7)|((blink&1)<<3)),page);
		if(x>=(width-1)) {
			if(y<gmaxrow-1)
				++y;
			curset(y,0,page);
		} else
			curset( y,++x,page );
	}
}
