/*
 * strmove.c
 * contains: strmove(),strnull(),strover(),strtbig()
 *
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gfuncts.h"


/*
 *  int
 * strmove(to,from)
 *
 * ARGUMENT
 *  (char *)	to	-	points to destination
 *  (char *)	from	-	points to source
 *
 * DESCRIPTION
 *  This function moves a string, the source string is moved in either
 *  direction and may be partially overlaid on itself.
 *
 * RETURNS
 *  relative distance of the move, 0 if destination offset is the same
 *  as the source, 0 if memory cannot be allocated.
 */
int GF_CONV strmove(to,from)
char *to,*from;
{
	char *pi1,*ps,*pd;
	int dist,n;

	ps=from;
	pd=to;
	n=strlen(from)+1;
	if((pi1=malloc((unsigned)(n+2)))==NULL)
		return 0;
	strcpy(pi1,from);
	strcpy(to,pi1);
	free(pi1);
	dist=0;
	if(ps==pd) 
		return 0;
	else if(ps>pd) 
		while(ps-->pd) 
			dist--;
	else if(ps<pd) 
		while (ps++<pd)
			dist++;
	return dist;
}

/*
 *  bool
 * strnull(string)
 *
 * ARGUMENT
 *  (char *)	string	-	points to string to test
 *
 * DESCRIPTION
 *  determines if string is a null string.
 *
 * RETURNS
 *  TRUE if null string, FALSE if not null string.
 */
bool GF_CONV strnull(str)
char *str;
{
	return(strlen(str)==0?TRUE:FALSE);
}

/*
 *  char *
 * strover(string,n)
 *
 * ARGUMENT
 *  (char *)	string	-	source string to move.
 *  (int)	n	-	Number of character positions to move
 *
 * DESCRIPTION
 *  Move a string of n characters (usually over itself)
 *
 * RETURNS
 *  pointer to the destination string
 */
char* GF_CONV strover(str,n)
char *str;
int n;
{
	int GF_CONV strmove();
	char *pd,*p;

	if(n==0)
		return (char *)0;
	else {
		p=str;
		if(n>0) 
			while(n--) 
				--p;
		else 
			while(++n<=0) 
				++p;
		pd=p;
		strmove(p,str);
	}
	return(pd);
}

/*
 *  bool
 * strtbig(str,maxchars)
 *
 * ARGUMENT
 *  (char *)	str	-	points to string to test
 *  (int)	maxchars -	length to test for.
 *
 * DESCRIPTION
 *  compare the length of str with maxchars.
 *
 * RETURNS
 *  Return TRUE if str is longer than maxchars, else return FALSE
 */
bool GF_CONV strtbig(str,maxchars)
char *str;
int maxchars;
{
	return((bool) (strlen(str) > (unsigned)maxchars));
}
