/*
 * strverif.c
 * contains: strverify()
 *
 *   Copyright (C)1987-1990 Greenleaf Software Inc. All Rights Reserved.
 */

#include <stdio.h>
#include <string.h>
#include "gfuncts.h"

/*
 *  bool
 * strverify(str,test,option)
 *
 * ARGUMENT
 *  (char *)	str		-	string to check
 *  (char *)	test		-	test characters
 *  (int)	option		-	options, they are:
 *			1 Return TRUE if all chars in test are also in str.
 *			2 Return TRUE if all chars in test are in str,
 *			  case insensitive.
 *			3 Return TRUE if at least one char in test is found
 *			  in str.
 *			4 Return TRUE if no chars not in test are found in
 *			  str.
 *
 * DESCRIPTION
 *  This function checks str to see if the characters from test are contained
 *  in it.
 *
 * RETURNS
 *  1/0 (bool) according to option
 *
 * AUTHOR
 *  ""   17-APR-1987  09:35:09.50
 *
 * MODIFICATIONS
 *  Ruby Crawford	  23-OCT-1989
 *   _checkstring() returned incorrect value; changed to return TRUE if
 *   test value actually found.
 *  Ruby Crawford  	  20-OCT-1989
 *   retvalue was being checked against strlen(test) but test was being
 *	  incremented, thus causing this check to fail and the function to
 *	  return the incorrect value if cases 1 and 2 were found to be true;
 *   added temp to point to same location as test, preserving the strlen.
 */
bool GF_CONV strverify(str,test,option)
char *str,*test;
int option;
{
	int retvalue=0,GF_CONV _checkstring();
	char *temp;

	if(!str||!test||option<1||option>4)
		return(FALSE);
	temp = test;
	switch(option) {
		case 1:
		case 2:
			for(--option;*test;++test)
				retvalue+=_checkstring(*test,str,option);
			retvalue=(retvalue==(int)strlen(temp))?TRUE:FALSE;
			break;
		case 3:
			for(;*test;++test)
				retvalue+=_checkstring(*test,str,0);
			retvalue=(retvalue)?TRUE:FALSE;
			break;
		case 4:
			for(;*str;++str)
				retvalue+=_checkstring(*str,test,0);
			retvalue=(retvalue==(int)strlen(str))?TRUE:FALSE;
			break;
	}
	return((bool)retvalue);
}

/*
 *  int
 * _checkstring(c,s,opt)
 *
 * ARGUMENT
 *  (char)	c	-	character to check for
 *  (char *)	s	-	string to check
 *  (int)	opt	-	option, 0=don't ignore case
 *					1=ignore case
 *
 * DESCRIPTION
 *  This is a helper function for strverify().  It checks a string for
 *  presence of a character.  The opt parameter controls whether or not
 *  the search is case sensitive or not.
 *
 * RETURNS
 *  1=character found, 0=character not found
 *
 * AUTHOR
 *  ""   17-APR-1987  10:08:36.08
 */
int GF_CONV _checkstring(c,s,opt)
char c,*s;
int opt;
{
	char GF_CONV _gtoupper();
	
	if(!opt)
		while(*s&&*s!=c)
			++s;
	else {
		c=_gtoupper(c);
		while(*s&&_gtoupper(*s)!=c)
			++s;
	}
	return((*s)?1:0);
}
